from .types import Discovery

__all__ = ("starting_discoveries",)

starting_discoveries: list[Discovery] = [
    {
        "name": "Water",
        "emoji": "💧",
        "is_first_discovery": False,
    },
    {
        "name": "Fire",
        "emoji": "🔥",
        "is_first_discovery": False,
    },
    {
        "name": "Wind",
        "emoji": "🌬️",
        "is_first_discovery": False,
    },
    {
        "name": "Earth",
        "emoji": "🌍",
        "is_first_discovery": False,
    },
]
